<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package AVIF Image Optimizer
 */

namespace Amasty\AvifImageOptimizer\Model\Command;

use Amasty\AvifImageOptimizer\Model\ConfigProvider;
use Amasty\ImageOptimizer\Api\Data\QueueInterface;
use Amasty\ImageOptimizer\Model\Command\CommandInterface;
use Amasty\ImageOptimizer\Model\Command\Utils\RemoteStorageProcessor;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteInterface;
use Magento\Framework\Filesystem\DriverPool;
use Magento\Framework\Filesystem\Io\File;
use Psr\Log\LoggerInterface;

class Cavif implements CommandInterface
{
    public const JPG = 'jpg';
    public const JPEG = 'jpeg';
    public const PNG = 'png';

    /**
     * @var WriteInterface
     */
    private $mediaDirectory;

    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @var RemoteStorageProcessor
     */
    private $remoteStorageProcessor;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var Filesystem
     */
    private $filesystem;

    /**
     * @var File
     */
    private $file;

    public function __construct(
        Filesystem $filesystem,
        ConfigProvider $configProvider,
        RemoteStorageProcessor $remoteStorageProcessor,
        LoggerInterface $logger,
        File $file
    ) {
        $this->filesystem = $filesystem;
        $this->configProvider = $configProvider;
        $this->remoteStorageProcessor = $remoteStorageProcessor;
        $this->logger = $logger;
        $this->file = $file;
    }

    public function getName(): string
    {
        return (string)__('Cavif');
    }

    public function getType(): string
    {
        return 'cavif';
    }

    public function run(QueueInterface $queue, string $inputFile, string $outputFile = ''): void
    {
        try {
            if ($this->remoteStorageProcessor->isRemoteStorageEnabled()) {
                $this->remoteStorageProcessor->copyFromRemote($inputFile);
                $this->generateAvif($queue, $inputFile, $outputFile);

                $resultFile = $outputFile ?: $inputFile;
                $this->remoteStorageProcessor->copyToRemote($resultFile);

                $this->getMediaDirectory()->delete($inputFile);
                $this->getMediaDirectory()->delete($resultFile);
            } else {
                $this->generateAvif($queue, $inputFile, $outputFile);
            }
        } catch (\Exception $e) {
            $this->logger->error($e);
        }
    }

    public function isAvailable(): bool
    {
        $requiredVersion = '8.1.0';
        $currentVersion = phpversion();

        return (bool)version_compare($currentVersion, $requiredVersion, '>=');
    }

    public function getUnavailableReason(): string
    {
        return __('PHP version is not compatible. In order to support the Avif format PHP 8.1 version or higher' .
            ' is required')->getText();
    }

    private function generateAvif(QueueInterface $queue, string $inputFile, string $outputFile): void
    {
        $inputFile = $this->getMediaDirectory()->getAbsolutePath($inputFile);
        $pathInfo = $this->file->getPathInfo($inputFile);
        if (!isset($pathInfo['extension'])) {
            return;
        }

        switch ($pathInfo['extension']) {
            case self::JPG:
            case self::JPEG:
                //phpcs:ignore Magento2.Functions.DiscouragedFunction.Discouraged
                $image = imagecreatefromjpeg($inputFile);
                break;
            case self::PNG:
                //phpcs:ignore Magento2.Functions.DiscouragedFunction.Discouraged
                $image = imagecreatefrompng($inputFile);
                break;
            default:
                return;
        }

        $outputFile = str_replace(
            $queue->getFilename(),
            $outputFile,
            $inputFile
        );

        //phpcs:ignore Magento2.Functions.DiscouragedFunction.Discouraged
        imageavif(
            $image,
            $outputFile,
            $this->configProvider->getAvifCompressionQuality(),
            $this->configProvider->getAvifEncodingSpeed()
        );
    }

    private function getMediaDirectory(): WriteInterface
    {
        if ($this->mediaDirectory === null) {
            $this->mediaDirectory = $this->filesystem->getDirectoryWrite(
                DirectoryList::MEDIA,
                DriverPool::FILE
            );
        }

        return $this->mediaDirectory;
    }
}
