<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package AVIF Image Optimizer
 */

namespace Amasty\AvifImageOptimizer\Model\ImageProcessor;

use Amasty\ImageOptimizer\Api\Data\ImageSettingInterface;
use Amasty\ImageOptimizer\Api\Data\QueueInterface;
use Amasty\ImageOptimizer\Model\Command\CommandProvider;
use Amasty\ImageOptimizer\Model\ImageProcessor\ImageProcessorInterface;
use Amasty\PageSpeedTools\Model\OptionSource\Resolutions;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteInterface;
use Magento\Framework\Filesystem\DriverPool;

class CreateAvif implements ImageProcessorInterface
{
    /**
     * @var CommandProvider
     */
    private $avifCommandProvider;

    /**
     * @var Filesystem
     */
    private $filesystem;

    /**
     * @var WriteInterface
     */
    private $mediaDirectory;

    /**
     * @var array
     */
    private $availableTools = [];

    public function __construct(
        CommandProvider $avifCommandProvider,
        Filesystem $filesystem
    ) {
        $this->avifCommandProvider = $avifCommandProvider;
        $this->filesystem = $filesystem;
    }

    public function process(QueueInterface $queue): void
    {
        if (!$queue->getAvifTool()) {
            return;
        }

        $avifCommand = $this->avifCommandProvider->get($queue->getAvifTool());
        $filename = (string)$queue->getFilename();

        $avifCommand->run(
            $queue,
            $filename,
            $this->getAvifFileName($filename, $queue)
        );
    }

    public function prepareQueue(string $file, ImageSettingInterface $imageSetting, QueueInterface $queue): bool
    {
        if (!$imageSetting->getAvifTool()) {
            return false;
        }
        if (!isset($this->availableTools[$imageSetting->getAvifTool()])) {
            try {
                $this->availableTools[$imageSetting->getAvifTool()] = $this->avifCommandProvider
                    ->get($imageSetting->getAvifTool())
                    ->isAvailable();
            } catch (LocalizedException $e) {
                $this->availableTools[$imageSetting->getAvifTool()] = false;
            }
        }

        if (!$this->availableTools[$imageSetting->getAvifTool()]) {
            $queue->setAvifTool('');

            return false;
        }
        $queue->setAvifTool($imageSetting->getAvifTool());

        return true;
    }

    private function getMediaDirectory(): WriteInterface
    {
        if ($this->mediaDirectory === null) {
            $this->mediaDirectory = $this->filesystem->getDirectoryWrite(
                DirectoryList::MEDIA,
                DriverPool::FILE
            );
        }

        return $this->mediaDirectory;
    }

    private function getAvifFileName(string $imagePath, QueueInterface $queue): string
    {
        $avifPath = str_replace(
            $queue->getFilename(),
            Resolutions::AVIF_DIR . $queue->getFilename(),
            $imagePath
        );
        if (!$this->getMediaDirectory()->isExist($this->dirname($avifPath))) {
            $this->getMediaDirectory()->create($this->dirname($avifPath));
        }

        return str_replace(
            '.' . $queue->getExtension(),
            '_' . $queue->getExtension() . '.avif',
            $avifPath
        );
    }

    private function dirname(string $file): string
    {
        //phpcs:ignore
        return dirname($file);
    }
}
